package com.taauav.front.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.taauav.admin.entity.SysAdmin;
import com.taauav.common.bean.Response;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import com.taauav.front.entity.UserAdmin;
import com.taauav.front.service.IUserAdminService;
import com.taauav.front.service.IUserAuthRuleService;
import com.taauav.front.vo.UserAdminVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/front/user")
public class FrontUserController {

    @Autowired
    private IUserAdminService adminService;
    @Autowired
    private IUserAuthRuleService userAuthRuleService;
    @Autowired
    private Response response;

    /**
     * 登录用户的信息
     *
     * @return
     */
    @RequestMapping("/info")
    public Response info() {
        Integer adminId = ShiroUtils.getAdminId();
        UserAdminVo info = adminService.getAdminInfo(adminId);
        Map<String, Object> map = new HashMap<String, Object>();
        List<Map<String, Object>> ruleList = userAuthRuleService.getRuleList(info.getId());
        String permission = userAuthRuleService.getPermission(info);
        map.put("info", info);
//        map.put("ruleList",ruleList);
        map.put("permission", permission);
        return response.success(map);
    }

    /**
     * 获取所有菜单
     *
     * @return
     */
    @GetMapping("/authList")
    public Response authList() {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("mark", 1);
        wrapper.eq("status", 1);
        List<Map<String, Object>> list = userAuthRuleService.listMaps(wrapper);
        return response.success(list);
    }

    /**
     * 修改密码接口
     *
     * @param map
     * @return
     */
    @PostMapping("/editPassword")
    public Response editPassword(@RequestBody Map<String, String> map) {
        if (map == null) {
            return response.failure("参数不能为空");
        }
        String adminId = "id";
        if (!map.containsKey(adminId) || "".equals(map.get(adminId))) {
            return response.failure("用户id不能为空");
        }
        UserAdmin admin = adminService.getById(Integer.valueOf(map.get(adminId)));
        if (admin == null) {
            return response.failure("用户信息不存在");
        }
        if (admin.getStatus() == 2) {
            return response.failure("用户已禁用，请联系管理员处理");
        }
        String oldPassword = "oldPassword";
        if (!map.containsKey(oldPassword) || "".equals(map.get(oldPassword))) {
            return response.failure("请输入原密码");
        }
        String oldPwd = FunctionUtils.password(map.get(oldPassword));
        if (!oldPwd.equals(admin.getPassword())) {
            return response.failure("原密码输入错误");
        }
        String newPwd = "password";
        if (!map.containsKey(newPwd) || "".equals(map.get(newPwd))) {
            return response.failure("请输入新密码");
        }
        String reNewPwd = "rePassword";
        if (!map.containsKey(reNewPwd) || "".equals(map.get(reNewPwd))) {
            return response.failure("请再次输入新密码");
        }
        if (!map.get(newPwd).equals(map.get(reNewPwd))) {
            return response.failure("两次密码输入不一致");
        }
        String newPassword = FunctionUtils.password(map.get(newPwd));
        if (newPassword.equals(admin.getPassword())) {
            return response.failure("新密码和原密码不能重复");
        }
        admin.setPassword(newPassword);
        adminService.updateById(admin);
        return response.success("操作成功");
    }

}
