package com.taauav.front.controller;

import com.taauav.admin.entity.TauvInspect;
import com.taauav.admin.service.ITauvDriverPointService;
import com.taauav.admin.service.ITauvInspectService;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.front.dto.IndexInspectDto;
import com.taauav.front.service.IUserInspectDriverService;
import com.taauav.front.service.IUserInspectService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * 指挥大屏 - 任务管理 控制器
 *
 * @author daixiantong
 * @date 2020-05-21
 */
@RestController
@RequestMapping("/index/inspect")
public class IndexInspectController {
    @Autowired
    private IUserInspectDriverService inspectDriverService;
    @Autowired
    private IUserInspectService inspectService;
    @Resource
    private Response response;

    /**
     * 统计任务数量
     * @return
     */
    @GetMapping("/count")
    public Response countInspect() {
        return inspectDriverService.countInspectNum();
    }

    /**
     * 任务管理列表
     * @return
     */
    @GetMapping("/list")
    public Response list(IndexInspectDto indexInspectDto) {
        return inspectDriverService.getInspectList(indexInspectDto);
    }

    /**
     * 新建任务
     * @param inspect
     * @param bindingResult
     * @return
     */
    @PostMapping("/addTask")
    public Response addTask(@RequestBody @Valid TauvInspect inspect , BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            List<FieldError> errorList = bindingResult.getFieldErrors();
            String msg = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(msg);
        }
        return inspectService.addTask(inspect);
    }
}
