package com.taauav.front.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.taauav.admin.entity.TauvDriver;
import com.taauav.admin.entity.TauvInspectQuestion;
import com.taauav.admin.mapper.TauvDriverMapper;
import com.taauav.common.bean.Response;
import com.taauav.common.util.StringUtils;
import com.taauav.front.mapper.UserInspectQuestionMapper;
import com.taauav.front.mapper.UserWaterAlarmInfoMapper;
import com.taauav.front.mapper.UserWaterAlarmMapper;
import com.taauav.front.query.UserDriverWaterAlarmQuery;
import com.taauav.front.query.UserMultiDriverWaterAlarmQuery;
import com.taauav.front.vo.inspectquestion.InspectQuestionCategoryVo;
import io.netty.util.internal.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.util.*;

@RestController
@RequestMapping("/front/data")
public class UserDataStatisticsController {

    @Autowired
    private UserInspectQuestionMapper inspectQuestionMapper;
    @Autowired
    private UserWaterAlarmInfoMapper userWaterAlarmInfoMapper;
    @Autowired
    private UserWaterAlarmMapper userWaterAlarmMapper;
    @Autowired
    private TauvDriverMapper driverMapper;
    @Autowired
    private Response response;

    @GetMapping("/index")
    public Response index() {
        // 巡检问题处理情况统计
        Map<String, Object> inspectQuestionInfo = getInspectQuestionInfo();
        // 获取月度巡检问题统计
        List<Map<String, Integer>> monthQuestionList = getMonthQuestionList();
        // 巡检问题类型统计
        List<InspectQuestionCategoryVo> inspectQuestionCategoryList = inspectQuestionMapper.getInspectQuestionCategoryList();
        // 获取水质警告分类统计列表
        List<Map<String, Object>> waterAlarmCategoryList = userWaterAlarmInfoMapper.getWaterAlarmCategoryList();

        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("inspectQuestionInfo", inspectQuestionInfo);
        result.put("monthQuestionList", monthQuestionList);
        result.put("inspectQuestionCategoryList", inspectQuestionCategoryList);
        result.put("waterAlarmCategoryList", waterAlarmCategoryList);
        return response.success(result);
    }

    /**
     * 巡检问题处理情况统计
     *
     * @return
     */
    private Map<String, Object> getInspectQuestionInfo() {
        Map<String, Object> result = new HashMap<>();
        // 获取已发现问题数
        Integer totalNum = inspectQuestionMapper.selectCount(new LambdaQueryWrapper<TauvInspectQuestion>()
                .eq(TauvInspectQuestion::getMark, 1));
        result.put("totalNum", totalNum);
        // 获取已解决问题数
        Integer finishedNum = inspectQuestionMapper.selectCount(new LambdaQueryWrapper<TauvInspectQuestion>()
                .eq(TauvInspectQuestion::getStatus, 3)
                .eq(TauvInspectQuestion::getMark, 1));
        result.put("finishedNum", finishedNum);
        // 未完成数
        result.put("waitNum", totalNum - finishedNum);

        // 已处理占比
        if (totalNum > 0) {
            result.put("percentage", String.format("%.2f", (float) finishedNum / (float) totalNum * 100));
        } else {
            result.put("percentage", "0.00");
        }
        return result;
    }

    /**
     * 月度巡检问题统计
     *
     * @return
     */
    private List<Map<String, Integer>> getMonthQuestionList() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar date = Calendar.getInstance();
        Integer year = Integer.valueOf(date.get(Calendar.YEAR));
        List<Map<String, Integer>> mapList = new ArrayList<>();
        for (int i = 1; i <= 12; i++) {
            Date beginTime = getBeginTime(year, i);
            System.out.println(sdf.format(beginTime));

            Date endTime = getEndTime(year, i);
            System.out.println(sdf.format(endTime));

            // 已完成数量
            QueryWrapper<TauvInspectQuestion> queryWrapper = new QueryWrapper<>();
            queryWrapper.between("create_time", sdf.format(beginTime), sdf.format(endTime));
            queryWrapper.eq("status", 3);
            queryWrapper.eq("mark", 1);
            Integer finishedNum = inspectQuestionMapper.selectCount(queryWrapper);

            QueryWrapper<TauvInspectQuestion> queryWrapper2 = new QueryWrapper<>();
            queryWrapper2.between("create_time", sdf.format(beginTime), sdf.format(endTime));
            queryWrapper2.le("status", 2);
            queryWrapper2.eq("mark", 1);
            Integer waitNum = inspectQuestionMapper.selectCount(queryWrapper2);

            Map<String, Integer> map = new HashMap<>();
            map.put("month", i);
            map.put("finishedNum", finishedNum);
            map.put("waitNum", waitNum);
            map.put("totalNum", finishedNum + waitNum);
            mapList.add(map);
        }
        return mapList;
    }

    public static Date getBeginTime(int year, int month) {
        YearMonth yearMonth = YearMonth.of(year, month);
        LocalDate localDate = yearMonth.atDay(1);
        LocalDateTime startOfDay = localDate.atStartOfDay();
        ZonedDateTime zonedDateTime = startOfDay.atZone(ZoneId.of("Asia/Shanghai"));

        return Date.from(zonedDateTime.toInstant());
    }

    public static Date getEndTime(int year, int month) {
        YearMonth yearMonth = YearMonth.of(year, month);
        LocalDate endOfMonth = yearMonth.atEndOfMonth();
        LocalDateTime localDateTime = endOfMonth.atTime(23, 59, 59, 999);
        ZonedDateTime zonedDateTime = localDateTime.atZone(ZoneId.of("Asia/Shanghai"));
        return Date.from(zonedDateTime.toInstant());
    }

    /**
     * 巡检问题河湖统计
     *
     * @return
     */
    @GetMapping("/getInspectQuestionDriverList")
    private Response getInspectQuestionDriverList(Integer type) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar date = Calendar.getInstance();
        if (type == 2) {
            // 前三个月
            String endTime = sdf.format(new Date());
            String startTime = addMonthDate(endTime, -3, "yyyy-MM-dd HH:mm:ss");
            List<Map<String, Object>> mapList = inspectQuestionMapper.getInspectQuestionDriverList(startTime, endTime);
            return response.success(mapList);
        } else if (type == 3) {
            // 前半年
            String endTime = sdf.format(new Date());
            String startTime = addMonthDate(endTime, -6, "yyyy-MM-dd HH:mm:ss");
            List<Map<String, Object>> mapList = inspectQuestionMapper.getInspectQuestionDriverList(startTime, endTime);
            return response.success(mapList);
        } else {
            // 当月
            Integer year = Integer.valueOf(date.get(Calendar.YEAR));
            Integer month = Integer.valueOf(date.get(Calendar.MONTH)) + 1;

            Date beginTime = getBeginTime(year, month);
            System.out.println(sdf.format(beginTime));

            Date endTime = getEndTime(year, month);
            System.out.println(sdf.format(endTime));

            List<Map<String, Object>> mapList = inspectQuestionMapper.getInspectQuestionDriverList(sdf.format(beginTime), sdf.format(endTime));
            return response.success(mapList);
        }
    }

    /**
     * 得到几个月之后的时间
     *
     * @param date
     * @param month
     * @return
     */
    public static String addMonthDate(String date, int month, String pattern) {
        String nowDate = null;
        SimpleDateFormat format = new SimpleDateFormat(pattern);
        try {
            Date parse = format.parse(date);
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(parse);
            calendar.add(Calendar.MONTH, month);
            nowDate = format.format(calendar.getTime());
        } catch (ParseException e) {
        }
        return nowDate;
    }

    /**
     * 获取水质污染程度统计
     *
     * @param type 类型：1当月 2三个月 3半年
     * @return
     */
    @GetMapping("/getWaterAlarmInfoList")
    private Response getWaterAlarmInfoList(Integer type) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar date = Calendar.getInstance();
        if (type == 2) {
            // 前三个月
            String endTime = sdf.format(new Date());
            String startTime = addMonthDate(endTime, -3, "yyyy-MM-dd HH:mm:ss");
            List<Map<String, Object>> mapList = userWaterAlarmMapper.getWaterAlarmInfoList(startTime, endTime);
            return response.success(mapList);
        } else if (type == 3) {
            // 前半年
            String endTime = sdf.format(new Date());
            String startTime = addMonthDate(endTime, -6, "yyyy-MM-dd HH:mm:ss");
            List<Map<String, Object>> mapList = userWaterAlarmMapper.getWaterAlarmInfoList(startTime, endTime);
            return response.success(mapList);
        } else {
            // 当月
            Integer year = Integer.valueOf(date.get(Calendar.YEAR));
            Integer month = Integer.valueOf(date.get(Calendar.MONTH)) + 1;
            Date beginTime = getBeginTime(year, month);
            Date endTime = getEndTime(year, month);
            List<Map<String, Object>> mapList = userWaterAlarmMapper.getWaterAlarmInfoList(sdf.format(beginTime), sdf.format(endTime));
            return response.success(mapList);
        }
    }

    /**
     * 获取单河流水质分析信息
     *
     * @return
     */
    @PostMapping("/getDriverWaterAlarmInfo")
    public Response getDriverWaterAlarmInfo(@RequestBody UserDriverWaterAlarmQuery query) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar date = Calendar.getInstance();
        // 获取当前年份
        Integer year = Integer.valueOf(date.get(Calendar.YEAR));

        // 统计分类
        List<String> stringList = new ArrayList<>();
        stringList.add("COD");
        stringList.add("NH3");
        stringList.add("BOD");
        stringList.add("TN");
        stringList.add("TP");

        List<Map<String, Object>> mapList = new ArrayList<>();
        for (int i = year - 4; i <= year; i++) {
            // 开始时间
            Date beginTime = getBeginTime(i, query.getMonth());
            // 结束时间
            Date endTime = getEndTime(i, query.getMonth());
            System.out.println("开始时间：" + sdf.format(beginTime));
            System.out.println("结束时间：" + sdf.format(endTime));

            for (String s : stringList) {
                Map<String, Object> map = new HashMap<>();
                map.put("driverId", query.getDriverId());
                map.put("startTime", sdf.format(beginTime));
                map.put("endTime", sdf.format(endTime));
                map.put("name", s);
                Map<String, Object> alarmInfo = userWaterAlarmMapper.getDriverWaterAlarmInfo(map);

                Map<String, Object> map1 = new HashMap<>();
                map1.put("month", String.format("%s.%s", i, query.getMonth()));
                map1.put("name", s);
                if (alarmInfo != null) {
                    map1.put("value", alarmInfo.get("value"));
                } else {
                    map1.put("value", 0);
                }
                mapList.add(map1);
            }
        }
        return response.success(mapList);
    }

    /**
     * 单河流环比分析
     *
     * @param query 参数
     * @return
     */
    @PostMapping("/getDriverWaterAlarmInfoList")
    public Response getDriverWaterAlarmInfoList(@RequestBody UserDriverWaterAlarmQuery query) {
        // 时间处理
        if (StringUtils.isEmpty(query.getTime())) {
            return response.failure("请选择时间段");
        }
        String[] time = query.getTime().split(" ~ ");
        String[] item = time[1].split("-");

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar date = Calendar.getInstance();
        Integer year = Integer.valueOf(item[0]);
        Integer month = Integer.valueOf(item[1]);
        Date beginTime = getBeginTime(year, month);
        Date endTime = getEndTime(year, month);

        // 统计分类
        List<String> stringList = new ArrayList<>();
        stringList.add("COD");
        stringList.add("NH3");
        stringList.add("BOD");
        stringList.add("TN");
        stringList.add("TP");

        List<Map<String, Object>> mapList = new ArrayList<>();
        for (int i = month - 1; i >= month - 5; i--) {
            String startTimeStr = addMonthDate(sdf.format(beginTime), -i, "yyyy-MM-dd HH:mm:ss");
            String endTimeStr = addMonthDate(sdf.format(endTime), -i, "yyyy-MM-dd HH:mm:ss");
            System.out.println(startTimeStr);
            System.out.println(endTimeStr);

            for (String s : stringList) {
                Map<String, Object> map = new HashMap<>();
                map.put("driverId", query.getDriverId());
                map.put("startTime", startTimeStr);
                map.put("endTime", endTimeStr);
                map.put("name", s);
                Map<String, Object> alarmInfo = userWaterAlarmMapper.getDriverWaterAlarmInfo(map);

                // 获取日期
                String[] timeArr = startTimeStr.split(" ");
                String[] dateArr = timeArr[0].split("-");

                Map<String, Object> map1 = new HashMap<>();
                map1.put("month", Integer.valueOf(dateArr[1]));
                map1.put("name", s);
                if (alarmInfo != null) {
                    map1.put("value", alarmInfo.get("value"));
                } else {
                    map1.put("value", 0);
                }
                mapList.add(map1);
            }
        }
        return response.success(mapList);
    }

    /**
     * 获取单河流水质分析信息
     *
     * @return
     */
    @PostMapping("/getMultiDriverWaterAlarmInfo")
    public Response getMultiDriverWaterAlarmInfo(@RequestBody UserMultiDriverWaterAlarmQuery query) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar date = Calendar.getInstance();
        // 获取当前年份
        Integer year = Integer.valueOf(date.get(Calendar.YEAR));
        // 河流ID
        String[] strings = query.getDriverId().split(",");
        List<Map<String, Object>> mapList = new ArrayList<>();
        for (int i = year - 4; i <= year; i++) {
            // 开始时间
            Date beginTime = getBeginTime(i, query.getMonth());
            // 结束时间
            Date endTime = getEndTime(i, query.getMonth());
            System.out.println("开始时间：" + sdf.format(beginTime));
            System.out.println("结束时间：" + sdf.format(endTime));

            for (String string : strings) {
                // 河湖ID
                Integer driverId = Integer.valueOf(string);
                // 获取河湖信息
                TauvDriver driver = driverMapper.selectById(driverId);
                if (driver == null) {
                    continue;
                }
                Map<String, Object> map = new HashMap<>();
                map.put("driverId", driverId);
                map.put("startTime", sdf.format(beginTime));
                map.put("endTime", sdf.format(endTime));
                map.put("name", query.getName());
                Map<String, Object> alarmInfo = userWaterAlarmMapper.getMultiDriverWaterAlarmInfo(map);
                Map<String, Object> map1 = new HashMap<>();
                map1.put("month", String.format("%s.%s", i, query.getMonth()));
                map1.put("name", driver.getName());
                if (alarmInfo != null) {
                    map1.put("value", alarmInfo.get("value"));
                } else {
                    map1.put("value", 0);
                }
                mapList.add(map1);
            }
        }
        return response.success(mapList);
    }

    /**
     * 单河流环比分析
     *
     * @param query 参数
     * @return
     */
    @PostMapping("/getMultiDriverWaterAlarmInfoList")
    public Response getMultiDriverWaterAlarmInfoList(@RequestBody UserMultiDriverWaterAlarmQuery query) {
        // 时间处理
        if (StringUtils.isEmpty(query.getTime())) {
            return response.failure("请选择时间段");
        }
        String[] time = query.getTime().split(" ~ ");
        String[] item = time[1].split("-");

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar date = Calendar.getInstance();
        Integer year = Integer.valueOf(item[0]);
        Integer month = Integer.valueOf(item[1]);
        Date beginTime = getBeginTime(year, month);
        Date endTime = getEndTime(year, month);

        // 河流ID
        String[] strings = query.getDriverId().split(",");

        List<Map<String, Object>> mapList = new ArrayList<>();
        for (int i = month - 1; i >= month - 5; i--) {
            String startTimeStr = addMonthDate(sdf.format(beginTime), -i, "yyyy-MM-dd HH:mm:ss");
            String endTimeStr = addMonthDate(sdf.format(endTime), -i, "yyyy-MM-dd HH:mm:ss");
            System.out.println(startTimeStr);
            System.out.println(endTimeStr);

            for (String string : strings) {
                // 河湖ID
                Integer driverId = Integer.valueOf(string);
                // 获取河湖信息
                TauvDriver driver = driverMapper.selectById(driverId);
                if (driver == null) {
                    continue;
                }
                Map<String, Object> map = new HashMap<>();
                map.put("driverId", driverId);
                map.put("startTime", startTimeStr);
                map.put("endTime", endTimeStr);
                map.put("name", query.getName());
                Map<String, Object> alarmInfo = userWaterAlarmMapper.getMultiDriverWaterAlarmInfo(map);

                // 获取日期
                String[] timeArr = startTimeStr.split(" ");
                String[] dateArr = timeArr[0].split("-");

                Map<String, Object> map1 = new HashMap<>();
                map1.put("month", Integer.valueOf(dateArr[1]));
                map1.put("name", driver.getName());
                if (alarmInfo != null) {
                    map1.put("value", alarmInfo.get("value"));
                } else {
                    map1.put("value", 0);
                }
                mapList.add(map1);
            }
        }
        return response.success(mapList);
    }
}
