package com.taauav.front.controller;


import com.taauav.admin.entity.SysCity;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.front.entity.UserDep;
import com.taauav.front.service.IUserDepService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * <p>
 * 部门管理表 前端控制器
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-25
 */
@RestController
@RequestMapping("/front/userdep")
public class UserDepController extends FrontBaseController {
    @Autowired
    private IUserDepService userDepService;
    @Autowired
    private Response response;

    private static final String controllerName = "userdep";

    /**
     * 部门列表
     *
     * @return
     */
    @PostMapping("/index")
//    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index() {
        return userDepService.getList();
    }

    /**
     * 添加机构
     *
     * @param entity        实体对象
     * @param bindingResult
     * @return
     */
    @PostMapping("/add")
//    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody @Valid UserDep entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return userDepService.add(entity);
    }

    /**
     * 获取机构详情
     *
     * @param id 机构ID
     * @return
     */
    @GetMapping("/info")
//    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response info(Integer id) {
        return userDepService.info(id);
    }

    /**
     * 编辑机构
     *
     * @param entity        实体对象
     * @param bindingResult
     * @return
     */
    @PostMapping("/edit")
//    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody @Valid UserDep entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return userDepService.edit(entity);
    }

    /**
     * 删除机构
     *
     * @param id 机构ID
     * @return
     */
    @PostMapping("/drop")
//    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response drop(Integer id) {
        return userDepService.delete(id);
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @PostMapping("/setStatus")
//    @RequiresPermissions(controllerName + ":" + PermissionConstants.STATUS_PERMISSION)
    public Response setStatus(@RequestBody UserDep entity) {
        return userDepService.setStatus(entity);
    }
}
