package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.entity.TauvInspectQuestion;
import com.taauav.admin.entity.TauvWaterAlarm;
import com.taauav.admin.entity.TauvWaterAlarmInfo;
import com.taauav.admin.service.ISysCityService;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import com.taauav.common.util.StringUtils;
import com.taauav.front.constant.UserWaterAlarmConstant;
import com.taauav.front.dto.WaterAlarmAssignDto;
import com.taauav.front.mapper.UserInspectQuestionMapper;
import com.taauav.front.mapper.UserWaterAlarmMapper;
import com.taauav.front.query.UserWaterAlarmQuery;
import com.taauav.front.service.IUserWaterAlarmInfoService;
import com.taauav.front.service.IUserWaterAlarmService;
import com.taauav.front.vo.wateralarm.UserWaterAlarmInfoListVo;
import com.taauav.front.vo.wateralarm.UserWaterAlarmInfoVo;
import com.taauav.front.vo.wateralarm.UserWaterAlarmListVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.*;

/**
 * <p>
 * 水质报警 服务实现类
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-15
 */
@Service
public class UserWaterAlarmServiceImpl extends BaseServiceImpl<UserWaterAlarmMapper, TauvWaterAlarm> implements IUserWaterAlarmService {

    @Autowired
    private ISysCityService sysCityService;

    @Autowired
    private UserWaterAlarmMapper userWaterAlarmMapper;

    @Autowired
    private Response response;

    @Autowired
    private IUserWaterAlarmInfoService userWaterAlarmInfoService;
    @Autowired
    private UserInspectQuestionMapper userInspectQuestionMapper;

    /**
     * 获取水质报警列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(UserWaterAlarmQuery query) {
        // 查询条件
        QueryWrapper<TauvWaterAlarm> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("mark", 1);
        // 报警编号
        if (!StringUtils.isEmpty(query.getAlarmSn())) {
            queryWrapper.like("alarm_sn", query.getAlarmSn());
        }
        // 区划ID
        if (query.getDriverArea() != null) {
            // 获取所有自己区划ID
            List<BigInteger> driverAreaList = new ArrayList<>();
            driverAreaList.add(query.getDriverArea());
            List<BigInteger> cityIdsList = sysCityService.getChildCityIds(query.getDriverArea());
            if (!cityIdsList.isEmpty()) {
                cityIdsList.forEach(item -> {
                    driverAreaList.add(item);
                });
            }
            queryWrapper.in("driver_area", driverAreaList);
        }
        // 河湖名称
        if (!StringUtils.isEmpty(query.getDriverName())) {
            queryWrapper.like("driver_name", query.getDriverName());
        }
        // 状态
        if (query.getStatus() != null && query.getStatus() > 0) {
            queryWrapper.eq("status", query.getStatus());
        }
        // 报警时间
        if (!StringUtils.isEmpty(query.getAlarmTime())) {
            String startTime = FunctionUtils.formatTime(Integer.valueOf(query.getAlarmTime()), "yyyy-MM-dd") + " 0:00:00";
            String endTime = FunctionUtils.formatTime(Integer.valueOf(query.getAlarmTime()), "yyyy-MM-dd") + " 23:59:59";
            queryWrapper.between("alarm_time", startTime, endTime);
        }
        IPage<TauvWaterAlarm> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<TauvWaterAlarm> data = userWaterAlarmMapper.selectPage(page, queryWrapper);
        List<TauvWaterAlarm> waterAlarmList = data.getRecords();
        List<UserWaterAlarmListVo> waterAlarmListVoList = new ArrayList<>();
        if (!waterAlarmList.isEmpty()) {
            waterAlarmList.forEach(item -> {
                UserWaterAlarmListVo waterAlarmListVo = new UserWaterAlarmListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, waterAlarmListVo);
                // 区划名称
                String cityName = sysCityService.getCityName(item.getDriverArea());
                waterAlarmListVo.setDriverAreaName(cityName);
                // 状态
                if (waterAlarmListVo.getStatus() != null && waterAlarmListVo.getStatus() > 0) {
                    waterAlarmListVo.setStatusName(UserWaterAlarmConstant.USER_WATER_ALARM_STATUS_LIST.get(waterAlarmListVo.getStatus()));
                }
                waterAlarmListVoList.add(waterAlarmListVo);
            });
        }
        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", waterAlarmListVoList);
        return response.success(result);
    }

    /**
     * 获取水质报警数据详情
     *
     * @param id 记录ID
     * @return
     */
    @Override
    public Response detail(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("报警记录ID不能为空");
        }
        TauvWaterAlarm entity = userWaterAlarmMapper.selectById(id);
        if (entity == null) {
            return response.failure("水质报警信息不存在");
        }
        // 拷贝熟悉
        UserWaterAlarmInfoVo userWaterAlarmInfoVo = new UserWaterAlarmInfoVo();
        BeanUtils.copyProperties(entity, userWaterAlarmInfoVo);

        // 获取水质报警信息列表
        List<UserWaterAlarmInfoListVo> waterAlarmInfoList = userWaterAlarmInfoService.getWaterAlarmInfoList(id);
        userWaterAlarmInfoVo.setWaterAlarmInfoList(waterAlarmInfoList);
        return response.success(userWaterAlarmInfoVo);
    }

    /**
     * 取消报警
     *
     * @param ids 记录ID
     * @return
     */
    @Override
    public Response cancel(String ids) {
        if (StringUtils.isEmpty(ids)) {
            return response.failure("报警记录ID不能为空");
        }
        if (ids.contains(",")) {
            Integer totalNum = 0;
            String[] strings = ids.split(",");
            for (String string : strings) {
                TauvWaterAlarm entity = userWaterAlarmMapper.selectById(ids);
                if (entity == null) {
                    continue;
                }
                if (entity.getStatus() == 3) {
                    continue;
                } else if (entity.getStatus() == 5) {
                    continue;
                }
                entity.setStatus(3);
                boolean result = this.editData(entity);
                if (result) {
                    totalNum++;
                }
            }
            return response.success(String.format("本次共取消【%s】条报警数据", totalNum));
        } else {
            TauvWaterAlarm entity = userWaterAlarmMapper.selectById(ids);
            if (entity == null) {
                return response.failure("水质报警信息不存在");
            }
            if (entity.getStatus() == 3) {
                return response.failure("水质报警已取消");
            } else if (entity.getStatus() == 5) {
                return response.failure("水质报警已完成,无法取消");
            }
            entity.setStatus(3);
            boolean result = this.editData(entity);
            if (!result) {
                return response.failure("操作失败");
            }
            return response.success("本次共取消【1】条报警数据");
        }
    }

    /**
     * 获取警告信息列表
     *
     * @param num 获取条数
     * @return
     */
    @Override
    public Response getWaterAlarmList(Integer num) {
        QueryWrapper<TauvWaterAlarm> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("status", 2);
        queryWrapper.eq("mark", 1);
        List<TauvWaterAlarm> waterAlarmList = userWaterAlarmMapper.selectList(queryWrapper);
        return response.success(waterAlarmList);
    }

    /**
     * 水质报警指派
     *
     * @param waterAlarmAssignDto 参数
     * @return
     */
    @Override
    public Response assign(WaterAlarmAssignDto waterAlarmAssignDto) {
        if (waterAlarmAssignDto.getWaterAlarmId() == null || waterAlarmAssignDto.getWaterAlarmId() <= 0) {
            return response.failure("水质警告ID不能为空");
        }
        if (waterAlarmAssignDto.getAssignUser() == null || waterAlarmAssignDto.getAssignUser() <= 0) {
            return response.failure("被指派责任人不能为空");
        }
        // 创建巡检问题
        TauvWaterAlarm waterAlarm = userWaterAlarmMapper.selectById(waterAlarmAssignDto.getWaterAlarmId());
        if (waterAlarm == null) {
            return response.failure("水质警告信息不存在");
        }
        // 设置状态为处理中
        waterAlarm.setStatus(4);
        Integer result = userWaterAlarmMapper.updateById(waterAlarm);
        if (result == 0) {
            return response.failure("水质报警状态更新失败");
        }
        TauvInspectQuestion entity = new TauvInspectQuestion();
        entity.setInspectDriverId(waterAlarm.getInspectDriverId());
        entity.setQuestionNo(createQuestionNo());
        // 问题类型：按照相关水体标准，监测段面水质不达标 by 朱志刚
        entity.setQuestionId(13);
        entity.setAssignUser(waterAlarmAssignDto.getAssignUser());
        entity.setAssignContact(waterAlarmAssignDto.getAssignContact());
        entity.setAssignNote(waterAlarmAssignDto.getAssignNote());
        entity.setAssignTime(DateUtil.now());
        entity.setStatus(2);
        entity.setCreateUser(ShiroUtils.getAdminId());
        entity.setCreateTime(DateUtil.now());
        Integer questionId = userInspectQuestionMapper.insert(entity);
        if (questionId == 0) {
            return response.failure("指派失败");
        }
        return response.success("指派成功");
    }

    /**
     * 创建问题编号
     *
     * @return
     */
    private String createQuestionNo() {
        Calendar date = Calendar.getInstance();
        String year = String.valueOf(date.get(Calendar.YEAR)).substring(2);
        String month = String.format("%02d", date.get(Calendar.MONTH) + 1);
        String code = "10" + year + month + (int) ((Math.random() * 9 + 1) * 100000);
        return code;
    }
}
