package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.entity.TauvWaterSpectrum;
import com.taauav.admin.service.ISysCityService;
import com.taauav.admin.service.ITauvDriverService;
import com.taauav.admin.vo.TauvDriverVo;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.RedisUtils;
import com.taauav.front.mapper.UserWaterSpectrumMapper;
import com.taauav.front.query.UserWaterSpectrumQuery;
import com.taauav.front.service.IUserWaterSpectrumService;
import com.taauav.front.vo.IndexDriverVo;
import com.taauav.front.vo.userwaterspectrum.UserWaterSpectrumListVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 多光谱图表 服务实现类
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-15
 */
@Service
public class UserWaterSpectrumServiceImpl extends BaseServiceImpl<UserWaterSpectrumMapper, TauvWaterSpectrum> implements IUserWaterSpectrumService {

    @Autowired
    private UserWaterSpectrumMapper userWaterSpectrumMapper;
    @Autowired
    private ISysCityService sysCityService;
    @Autowired
    private Response response;
    @Value("${server.UPLOAD_URL}")
    private String uploadUrl;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private ITauvDriverService driverService;

    /**
     * 获取多光谱数据列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(UserWaterSpectrumQuery query) {
        // 查询条件
        QueryWrapper<TauvWaterSpectrum> queryWrapper = new QueryWrapper<>();

        // 区划ID处理
        if (query.getDriverArea() != null) {
            List<BigInteger> cityIds = sysCityService.getChildCityIds(query.getDriverArea());
            // 加入当前选中的区划
            cityIds.add(query.getDriverArea());
            BigInteger[] integers = cityIds.toArray(new BigInteger[cityIds.size()]);
            queryWrapper.in("area_id", integers);
        }
        // 巡检时间处理
        if (!StringUtils.isEmpty(query.getInspectTime())) {
            String[] strings = query.getInspectTime().split(" ~ ");

            String startTime = strings[0] + " 0:00:00";
            String endTime = strings[1] + " 23:59:59";
            queryWrapper.between("inspect_time", startTime, endTime);
        }
        queryWrapper.eq("mark", 1);

        IPage<TauvWaterSpectrum> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<TauvWaterSpectrum> data = userWaterSpectrumMapper.selectPage(page, queryWrapper);
        List<TauvWaterSpectrum> waterSpectrumList = data.getRecords();
        List<UserWaterSpectrumListVo> waterSpectrumListVoList = new ArrayList<>();
        if (!waterSpectrumList.isEmpty()) {
            waterSpectrumList.forEach(item -> {
                // 拷贝属性
                UserWaterSpectrumListVo waterSpectrumListVo = new UserWaterSpectrumListVo();
                BeanUtils.copyProperties(item, waterSpectrumListVo);

                // 化学需氧量多光谱路径
                if (!StringUtils.isEmpty(item.getCodPic())) {
                    waterSpectrumListVo.setCodPic(uploadUrl + item.getCodPic());
                }
                // 氨氮多光谱路径
                if (!StringUtils.isEmpty(item.getNh3nPic())) {
                    waterSpectrumListVo.setNh3nPic(uploadUrl + item.getNh3nPic());
                }
                // 总磷多光谱路径
                if (!StringUtils.isEmpty(item.getTpPic())) {
                    waterSpectrumListVo.setTpPic(uploadUrl + item.getTpPic());
                }
                // 总氮多光谱路径
                if (!StringUtils.isEmpty(item.getTnPic())) {
                    waterSpectrumListVo.setTnPic(uploadUrl + item.getTnPic());
                }
                // 溶解氧多光谱路径
                if (!StringUtils.isEmpty(item.getDoPic())) {
                    waterSpectrumListVo.setDoPic(uploadUrl + item.getDoPic());
                }
                // 浊度多光谱路径
                if (!StringUtils.isEmpty(item.getTubPic())) {
                    waterSpectrumListVo.setTubPic(uploadUrl + item.getTubPic());
                }
                waterSpectrumListVoList.add(waterSpectrumListVo);
            });
        }

        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", waterSpectrumListVoList);
        return response.success(result);
    }

    /**
     * 获取指挥大屏 所有河湖最新多光谱图
     *
     * @return
     */
    @Override
    public Response getAllSpectrum() {
        List<TauvDriverVo> driverVoList = driverService.getDriverCacheList();
        List<IndexDriverVo> list = new ArrayList<>();
        if (driverVoList.size() > 0) {
            for (TauvDriverVo driverVo : driverVoList) {
                IndexDriverVo indexDriver = new IndexDriverVo();
                BeanUtils.copyProperties(driverVo, indexDriver, IndexDriverVo.class);
                TauvWaterSpectrum spectrum = baseMapper.selectLastSpectrumByDriver(driverVo.getId());
                String spectrumUrl = "";
                if (spectrum != null) {
                    if (!StringUtils.isEmpty(spectrum.getCodPic())) {
                        spectrumUrl = !spectrum.getCodPic().contains("http") ? uploadUrl + spectrum.getCodPic() : spectrum.getCodPic();
                    } else if (!StringUtils.isEmpty(spectrum.getNh3nPic())) {
                        spectrumUrl = !spectrum.getNh3nPic().contains("http") ? uploadUrl + spectrum.getNh3nPic() : spectrum.getNh3nPic();
                    } else if (!StringUtils.isEmpty(spectrum.getTpPic())) {
                        spectrumUrl = !spectrum.getTpPic().contains("http") ? uploadUrl + spectrum.getTpPic() : spectrum.getTpPic();
                    } else if (!StringUtils.isEmpty(spectrum.getTnPic())) {
                        spectrumUrl = !spectrum.getTnPic().contains("http") ? uploadUrl + spectrum.getTnPic() : spectrum.getTnPic();
                    } else if (!StringUtils.isEmpty(spectrum.getDoPic())) {
                        spectrumUrl = !spectrum.getDoPic().contains("http") ? uploadUrl + spectrum.getDoPic() : spectrum.getDoPic();
                    } else if (!StringUtils.isEmpty(spectrum.getTubPic())) {
                        spectrumUrl = !spectrum.getTubPic().contains("http") ? uploadUrl + spectrum.getTubPic() : spectrum.getTubPic();
                    }
                    indexDriver.setSpectrumUrl(spectrumUrl);
                    indexDriver.setLeftLatitude(spectrum.getLeftLatitude());
                    indexDriver.setLeftLongitude(spectrum.getLeftLongitude());
                    indexDriver.setRightLatitude(spectrum.getRightLatitude());
                    indexDriver.setRightLongitude(spectrum.getRightLongitude());
                }
                list.add(indexDriver);
            }
        }
        return response.success(list);
    }
}
